import matplotlib.pyplot as plt
import matplotlib.collections
import matplotlib.patches
import numpy as np
from matplotlib import rc
from errortools import *


rc('text', usetex=True)
rc('text.latex', preamble="\n".join([
    '\\usepackage{amsmath}',
    '\\newcommand{\\D}{\mathrm{d}}'
]))


def errorband(p, ax=None, underflow=False, overflow=False):
    if ax is None:
        ax = plt.gca()

    p = p.copy()

    delta = p[2,0] - p[1,0]

    if p[0,0] == -np.inf:
        if underflow is False:
            p = p[1:]
        else:
            s = 2 if type(underflow) == bool else underflow
            p[0,0] = p[1,0] - (s+0.5) * delta
    if p[-1,0] == +np.inf:
        if overflow is False:
            p = p[:-1]
        else:
            s = 2 if type(overflow) == bool else overflow
            p[-1,0] = p[-2,0] + (s+0.5) * delta

    boxes = [
        matplotlib.patches.Rectangle(
            (x - delta, y - e/2), delta, e
        )
        for x, y, e in p
    ]

    artist = ax.step(p[:,0], p[:,1])
    col = artist[0].get_color()

    pc = matplotlib.collections.PatchCollection(
        boxes,
        facecolor=col, edgecolor=col
    )

    ax.add_collection(pc)
    return artist


def twopanel(labelx,
             upleft=[], labupleft="",
             downleft=[], labdownleft="",
             upright=[], labupright="",
             downright=[], labdownright=""):

    if type(upleft) == np.ndarray:
        upleft = [upleft]
    if type(upright) == np.ndarray:
        upright = [upright]
    if type(downleft) == np.ndarray:
        downleft = [downleft]
    if type(downright) == np.ndarray:
        downright = [downright]

    fig, axs = plt.subplots(
        2, sharex=True, sharey=True, gridspec_kw={'hspace': 0}
    )
    axs[1].set_xlabel(labelx)

    axs[0].set_ylabel(labupleft)
    for i in upleft:
        errorband(i, ax=axs[0])

    if len(upright) > 0:
        ax2 = axs[0].twinx()
        if labupright is not None:
            ax2.set_ylabel(labupright)
        for i in upright:
            errorband(i, ax=ax2)

    axs[1].set_ylabel(labdownleft)
    for i in downleft:
        errorband(i, ax=axs[1])

    if len(downright) > 0:
        ax3 = axs[1].twinx()
        if labdownright is not None:
            ax3.set_ylabel(labdownright)
        for i, c in downright:
            errorband(i, ax=ax3)

    return fig, axs


def kplot(sigma, labelx='x_e', labelsigma=None,
          labelknlo="$K^{(1)}$", labelknnlo="$K^{(2)}$",
          show=[0,-1], showk=[1,2]):

    if labelsigma is None:
        labelsigma = r"$\D\sigma\,/\,\D " + labelx + "$"

    kwargs = {
        "labelx":"$%s$" % labelx,
        "labupleft": labelsigma
    }

    if type(sigma) != dict:
        raise ValueError("kplot takes dicts!")
    if "lo" not in sigma:
        raise KeyError("kplot: no leading order")
    if "nlo" not in sigma:
        raise KeyError("kplot: no next-to-leading order")
    if "nnlo" in sigma:
        orders = ['lo', 'nlo', 'nnlo']
        xsec = {
            'lo': sigma['lo'],
            'nlo': addplots(sigma['lo'], sigma['nlo']),
            'nnlo': addplots(
                sigma['lo'],
                addplots(sigma['nlo'], sigma['nnlo'])
            )
        }
    else:
        orders = ['lo', 'nlo']
        xsec = {
            'lo': sigma['lo'],
            'nlo': addplots(sigma['lo'], sigma['nlo'])
        }

    if 1 in showk:
        kwargs['downleft'] = divideplots(sigma['nlo'], sigma['lo'])
    if 2 in showk and 'nnlo' in sigma:
        kwargs['downright'] = divideplots(sigma['nnlo'], sigma['lo'])

    kwargs['upleft'] = [
        xsec[orders[i]] for i in show
    ]

    return twopanel(**kwargs)
